#!/usr/bin/env python

import os
import shutil
import sys

generic_folder = '/generic/'
generic_files = ['rules.dat',
                 'template_ppddl']

scenario_files = []
                  
problem_files = ['reward.dat',
                 'simulated_rules.dat',
                 'simulated_init_state.dat',
                 'rexd_config.cfg',]

optional_problem_files = ['simulated_init_state2.dat',
                          'simulated_init_state3.dat',
                          'simulated_init_state4.dat',
                          'simulated_init_state5.dat',
                          'finish_rule.dat',
                          'config',
                          'config_teacher',
                          'symbols.dat']
existing_files = []
                 
orig_copy_files = ['simulated_init_state.dat']
dest_copy_files = ['state_simulator.dat']
                  
generated_files = ['intellact_logging.txt',
                   'state.dat',
                   'gource.log',
                   'gource_caption.log',
                   'video_editing.avs',
                   'visualization.ass',
                   'visualization_verbose.ass',
                   'transitions_tmp.dat',
                   'transitions_full.dat',
                   'rules.ppddl',
                   'prada_reward.dat',
                   'prada_rules.dat',
                   'prada_state.dat',
                   'simulated_rules_simplified.dat']
                   
tmp_files = ['current_learned_rules.dat',
             'learned_rules.dat',
             'gen_symbols.dat',
             'learn.log',
             'learn.log.info',
             'MT.log',
             'info.html']


def load_scenario_and_problem(argv):
    scenario=""
    if len(sys.argv) <= 1:
        scenario="autas"
    else:
        scenario=sys.argv[1]
        
    problem=""
    if len(sys.argv) <= 2:
        res = scenario.split('/')
        if len(res) >= 2:
            scenario = res[0]
            problem = res[1]
        else:
            problem="det"
    else:
        problem=sys.argv[2]
    
    return [scenario, problem]


def start_experiment(scenario, problem):
    current_dir=os.getcwd()
    
    # COPY SCENARIO FILES
    scenario_files_dir=current_dir + '/' + scenario + '/'
    copy_files(scenario_files, scenario_files_dir, current_dir)
    
    # COPY PROBLEM FILES
    problem_files_dir=current_dir + '/' + scenario + '/' + problem + '/'
    copy_files(problem_files, problem_files_dir, current_dir)
    copy_files_optional(optional_problem_files, problem_files_dir, current_dir)
    
    # COPY GENERIC FILES
    generic_files_dir=current_dir + '/' + generic_folder + '/'
    copy_files(generic_files, generic_files_dir, current_dir)
    
    # GENERATE EMPTY REQUIRED FILES
    create_files(generated_files, current_dir)
    
    # CREATE COPIES
    copy_files_in_same_filefolder(orig_copy_files, dest_copy_files, current_dir)


def execute_experiment(scenario, problem):
    if os.path.exists('../bin/intellact'):
        os.system('../bin/intellact ')
    else:
        os.system('../../bin/intellact ')


def execute_debug_experiment(scenario, problem):
    if os.path.exists('../bin/intellact'):
        os.system('gdb -ex run --args ../bin/intellact ')
    else:
        os.system('gdb -ex run --args ../../bin/intellact ')


def finish_experiment(scenario, problem):
    current_dir=os.getcwd()
    experiments_dir=current_dir + '/experiments/'
    #experiment_type1="pasula"
    experiment_folder='experiment_' + scenario + '_' + problem + '_'

    ps=os.popen('ls '+experiments_dir+' | grep ' + experiment_folder + ' | cut -d "_" -f4 2>/dev/null')
    nums=ps.readlines()
    if len(nums):
        num=max([int(x) for x in nums])
    else:
        num=0
    num=num+1
    num_str = "%03d" % (num)
    
    copy_path=experiments_dir + '/' + experiment_folder + num_str
    os.mkdir(copy_path)
    print 'Moving to ' + copy_path
    
    # Add tmp rule files
    for files in (y for y in os.listdir(".") if (y.startswith("rules") and (y.endswith(".dat")) and (len(y)>=10)) ):
        generated_files.append(files)
    
    #try:
    move_files(scenario_files + generic_files + problem_files + existing_files + generated_files + dest_copy_files, current_dir, copy_path)
    move_files(optional_problem_files, current_dir, copy_path, verbose = False)
    remove_files(tmp_files, current_dir, verbose = False)
    #except:
        #print "Unexpected error:", sys.exc_info()[0]

    print "Finished."


def copy_files(files, orig_path, dest_path):
    for a_file in files:
        shutil.copy2(orig_path + '/' + a_file, dest_path)

def copy_files_optional(files, orig_path, dest_path):
    for a_file in files:
        try:
            shutil.copy2(orig_path + '/' + a_file, dest_path)
            existing_files.append(a_file)
        except:
            a=1 # noop

def copy_files_in_same_filefolder(orig_files, dest_files, folder):
    dest_files_copy = list(dest_files)
    for orig_file in orig_files:
        dest_file = dest_files_copy.pop(0)
        shutil.copy2(folder + '/' + orig_file, folder + '/' + dest_file)

def create_files(files, path):
    for a_file in files:
        open(path + '/' + a_file, 'w').close()

def move_files(files, orig_path, dest_path, verbose = True):
    for a_file in files:
        try:
            shutil.move(orig_path + '/' + a_file, dest_path)
        except:
            if verbose:
                print("Warning: Couldn't move " + a_file)

def remove_files(files, path, verbose = True):
    for a_file in files:
        try:
            os.remove(path + '/' + a_file)
        except:
            if verbose:
                print("Warning: Couldn't remove " + a_file)

    
def main():
    start_experiment()
    execute_experiment()

if __name__ == "__main__":
    main()
