#!/usr/bin/env bash

if [ $# -ge 1 ]; then
        PLOT=$1
else
        PLOT=1
fi
if [ $# -ge 2 ]; then
        SHOW_FIGURE=$2
else
        SHOW_FIGURE=0
fi

function reset_sums {
    cont=0
    beginning="=["
    actions_name="actions_${experiment_type}"
    teacher_name="teacher_${experiment_type}"
    planning_name="planning_${experiment_type}"
    learning_name="learning_${experiment_type}"
    failed_name="failed_${experiment_type}"
    explore_name="explore_${experiment_type}"
    noise_name="noise_${experiment_type}"
    dead_name="dead_${experiment_type}"
    sum_experiments="${actions_name}$beginning"
    sum_experiments2="${teacher_name}$beginning"
    sum_experiments3="${planning_name}$beginning"
    sum_experiments4="${learning_name}$beginning"
    sum_experiments5="${failed_name}$beginning"
    sum_experiments6="${explore_name}$beginning"
    sum_experiments7="${noise_name}$beginning"
    sum_experiments8="${dead_name}$beginning"
    if [[ -e $file_name ]]; then
        number_iterations=`cat $file_name | grep -i reset | wc -l`
    else
        number_iterations=0
    fi
}

function print_sums {
    ending="];"
#     if [ "$old_experiment_type" == "prob06" ]; then
#         number_iterations="20"
#     elif [ "$old_experiment_type" == "det" ]; then
#         number_iterations="20"
#     else
#         number_iterations="10"
#     fi
    echo "${sum_experiments}$ending"
    echo "${sum_experiments2}$ending"
    echo "${sum_experiments3}$ending"
    echo "${sum_experiments4}$ending"
    echo "${sum_experiments5}$ending"
    echo "${sum_experiments6}$ending"
    echo "${sum_experiments7}$ending"
    echo "${sum_experiments8}$ending"
    
    echo "% removing failed actions"
#     echo "${actions_name}=${actions_name}-${failed_name};"
#     echo "${actions_name}=${actions_name}-${teacher_name};"
#     echo "${actions_name}=${actions_name}-${noise_name};"
    echo "baseline = ones(1,${number_iterations}+2)*9;"
    
    echo "% var"
    echo "${actions_name}_v = var(${actions_name});"
    echo "${teacher_name}_v = var(${teacher_name});"
    echo "${planning_name}_v = var(${planning_name});"
    echo "${learning_name}_v = var(${learning_name});"
    echo "${failed_name}_v = var(${failed_name});"
    echo "${explore_name}_v = var(${explore_name});"
    echo "${noise_name}_v = var(${noise_name});"
    echo "${dead_name}_v = var(${dead_name});"
    
    echo "% mat"
    echo "${actions_name}_m = (${actions_name});"
    echo "${teacher_name}_m = (${teacher_name});"
    echo "${planning_name}_m = (${planning_name});"
    echo "${learning_name}_m = (${learning_name});"
    echo "${failed_name}_m = (${failed_name});"
    echo "${explore_name}_m = (${explore_name});"
    echo "${noise_name}_m = (${noise_name});"
    echo "${dead_name}_m = (${dead_name});"
    
    echo "% sum"
    echo "${actions_name} = sum(${actions_name}_m)/${cont};"
    echo "${teacher_name} = sum(${teacher_name}_m)/${cont};"
    echo "${planning_name} = sum(${planning_name}_m)/${cont};"
    echo "${learning_name} = sum(${learning_name}_m)/${cont};"
    echo "${failed_name} = sum(${failed_name}_m)/${cont};"
    echo "${explore_name} = sum(${explore_name}_m)/${cont};"
    echo "${noise_name} = sum(${noise_name}_m)/${cont};"
    echo "${dead_name} = sum(${dead_name}_m)/${cont};"

    if [ $SHOW_FIGURE -eq 1 ]; then
        echo "% figure"
        echo "action_bars = [${actions_name}', ${teacher_name}', ${failed_name}', ${explore_name}', ${noise_name}'];"
        echo "figure('name','${old_experiment_type}')"
        echo "y=1:1:${number_iterations};"
        echo "z=[0.5, y, (${number_iterations}+0.5)];"
        echo "bar(action_bars,'stacked');"
        echo "hold all"
        echo "plot(z,baseline,'w');"
        echo "hleg1 = legend('Actions','Teacher','Failed','Explore','Noise');"
    fi
}


function parse_experiment_logfile {
    n_resets=`cat $file_name |grep reset | wc -l`
    n_teacher=`cat $file_name |grep Teacher | wc -l`
    n_action=`cat $file_name |grep Action | wc -l`
    echo "Actions: ${n_action}(${n_teacher}) - Nª resets ${n_resets}"
    
    time_planning=`cat $file_name | grep 'Time planning' | cut -d\: -f2 | paste -sd+ | bc`
    time_learning=`cat $file_name | grep 'Time learning' | cut -d\: -f2 | paste -sd+ | bc`
    echo "Time planning: $time_planning | Time learning: $time_learning"

    failed_actions=`./count_valid_rules.py $file_name | paste -sd'\t'`
    action_per_iter=`awk  '$1 ~ /Action/ { count=count+1; } $1 ~ /Teacher/ { count_t=count_t+1; } ($1 ~ /reset/) || ($2 ~ /reset/) { if(count!="") { printf "%d (%d)\n", count, count_t; count=0; count_t=0; } } END { printf "%d (%d)\n", count, count_t; }' $file_name | paste -sd'\t'`
    planning_per_iter=`awk  '$2 ~ /planning/ { count=count+$3; } ($1 ~ /reset/) || ($2 ~ /reset/) { if(count!="") {print count; count=0; }; } END {print count; }' $file_name |paste -sd'\t'`
    learning_per_iter=`awk  '$2 ~ /learning/ { count=count+$3; } ($1 ~ /reset/) || ($2 ~ /reset/) { if(count!="") {print count; count=0; }; } END {print count; }' $file_name |paste -sd'\t'`
    indexes=`seq 10 | paste -sd'\t'`
    echo -e "Iteration         \t$indexes" 
    echo -e "Actions  per iter:\t$action_per_iter"
    echo -e "Failed   per iter:\t$failed_actions"
    echo -e "Planning per iter:\t$planning_per_iter"
    echo -e "Learning per iter:\t$learning_per_iter"
}


function parse_experiment_logfile_for_plotting {
    echo "%Matlab plots"
    cont=$(($cont + 1))
    
    if [ "$experiment_type" == "prob06" ]; then
        selected_experiments="(1:${number_iterations})"
    elif [ "$experiment_type" == "det" ]; then
        selected_experiments="(1:${number_iterations})"
    else
        selected_experiments="(1:${number_iterations})"
    fi

    action_per_iter=`awk  '$1 ~ /Action/ { count=count+1; } $1 ~ /Teacher/ { count_t=count_t+1; } ($1 ~ /reset/) || ($2 ~ /reset/) { if(count!="") { printf "%d\n", count; count=0; count_t=0; } } END { printf "%d\n", count; }' $file_name | paste -sd','`
    echo -e "actions${experiment_number}=[${action_per_iter}];"
    sum_experiments="${sum_experiments};actions${experiment_number}${selected_experiments}"

    action_per_iter=`awk  '$1 ~ /Action/ { count=count+1; } $1 ~ /Teacher/ { count_t=count_t+1; } ($1 ~ /reset/) || ($2 ~ /reset/) { if(count!="") { printf "%d\n", count_t; count=0; count_t=0; } } END { printf "%d\n", count_t; }' $file_name | paste -sd','`
    echo -e "teacher${experiment_number}=[${action_per_iter}];"
    sum_experiments2="${sum_experiments2};teacher${experiment_number}${selected_experiments}"

    planning_per_iter=`awk  '$2 ~ /planning/ { count=count+$3; } ($1 ~ /reset/) || ($2 ~ /reset/) { if(count!="") {print count; count=0; }; } END {print count; }' $file_name |paste -sd','`
    echo -e "planning${experiment_number}=[${planning_per_iter}];"
    sum_experiments3="${sum_experiments3};planning${experiment_number}${selected_experiments}"

    learning_per_iter=`awk  '$2 ~ /learning/ { count=count+$3; } ($1 ~ /reset/) || ($2 ~ /reset/) { if(count!="") {print count; count=0; }; } END {print count; }' $file_name |paste -sd','`
    echo -e "learning${experiment_number}=[${learning_per_iter}];"
    sum_experiments4="${sum_experiments4};learning${experiment_number}${selected_experiments}"
    
    failed_per_iter=`awk  '$1 ~ /Action/ { count=count+1; } $1 ~ /Failed/ { count_t=count_t+1; } ($1 ~ /reset/) || ($2 ~ /reset/) { if(count!="") { printf "%d\n", count_t; count=0; count_t=0; } } END { printf "%d\n", count_t; }' $file_name | paste -sd','`
    echo -e "failed${experiment_number}=[${failed_per_iter}];"
    sum_experiments5="${sum_experiments5};failed${experiment_number}${selected_experiments}"
    
    explore_per_iter=`awk  '$1 ~ /Action/ { count=count+1; } $1 ~ /Explore/ { count_t=count_t+1; } ($1 ~ /reset/) || ($2 ~ /reset/) { if(count!="") { printf "%d\n", count_t; count=0; count_t=0; } } END { printf "%d\n", count_t; }' $file_name | paste -sd','`
    echo -e "explore${experiment_number}=[${explore_per_iter}];"
    sum_experiments6="${sum_experiments6};explore${experiment_number}${selected_experiments}"
    
    noise_per_iter=`awk  '$1 ~ /Action/ { count=count+1; } $1 ~ /Noise/ { count_t=count_t+1; } ($1 ~ /reset/) || ($2 ~ /reset/) { if(count!="") { printf "%d\n", count_t; count=0; count_t=0; } } END { printf "%d\n", count_t; }' $file_name | paste -sd','`
    echo -e "noise${experiment_number}=[${noise_per_iter}];"
    sum_experiments7="${sum_experiments7};noise${experiment_number}${selected_experiments}"
    
    dead_per_iter=`awk  '$1 ~ /Action/ { count=count+1; } $1 ~ /Dead/ { count_t=count_t+1; } ($1 ~ /reset/) || ($2 ~ /reset/) { if(count!="") { printf "%d\n", count_t; count=0; count_t=0; } } END { printf "%d\n", count_t; }' $file_name | paste -sd','`
    echo -e "dead${experiment_number}=[${dead_per_iter}];"
    sum_experiments8="${sum_experiments8};dead${experiment_number}${selected_experiments}"
}

old_experiment_type="init"
reset_sums

for Y in `ls`
do

if [ -d "$Y" ]; then

for X in `ls $Y`
do
    if [ -d "$Y/$X" ]; then
        file_name="${Y}/${X}/intellact_logging.txt"
        if [ -e $file_name ]; then
            experiment_type=`echo $X | cut -d '_' -f 3`
            experiment_type="${Y}_${experiment_type}"
            experiment_number=`echo $X | cut -d '_' -f 4`
            if [ "$old_experiment_type" == "init" ]; then
                reset_sums
            elif [ "$experiment_type" != "$old_experiment_type" ]; then
                if [ $PLOT -eq 1 ]; then
                    print_sums
                fi
                echo "%--------------------------------------------------"
                reset_sums
            fi
            old_experiment_type=$experiment_type
            echo "%Pasula $experiment_type - $experiment_number"
            
            if [ $PLOT -eq 0 ]; then
                parse_experiment_logfile
            fi
            
            if [ $PLOT -eq 1 ]; then
                parse_experiment_logfile_for_plotting
            fi
            
            echo
        fi
    fi
done
fi
done

if [ $PLOT -eq 1 ]; then
    print_sums
fi

