/*
 * Robot simulator.
 * Copyright (C) 2014  David Martínez (dmartinez@iri.upc.edu)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#ifndef INTELLACT_SIMULATOR_H
#define INTELLACT_SIMULATOR_H

#include <boost/asio.hpp>

#include <symbolic/symbols.h>
#include <symbolic/predicates.h>
#include <symbolic/rules.h>
#include <symbolic/state.h>
#include <symbolic/scenario.h>
#include <rexd/config_reader.h>

using boost::asio::ip::tcp;

/**
 * \class Simulator
 * \brief A basic simulator based on NID rules.
 * 
 * Simulator for relational domains.
 */
class Simulator
{
public:
    // constructor
    Simulator(std::string host);
    
    
    void main_loop();

private:
    ConfigReader config_reader;
    ScenarioPtr scenario_ptr;
    tcp::iostream tcp_client;
    
    int num_resets; // needed only as info about completion
    float last_reward, accumulated_reward;
    
    void reset(std::string state_path);
    
    void reset_parser(std::string input_command);
    bool parse_command(std::string input_command);
    
    std::string get_state();
    std::string apply_action(Symbol action, int forced_outcome = -1);
};

#endif
