/*
 * IPPC planner wrapper.
 * Copyright (C) 2014  David Martínez (dmartinez@iri.upc.edu)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#ifndef INTELLACT_IPPC_PLANNER_H
#define INTELLACT_IPPC_PLANNER_H

#include <rexd/planner.h>

#include <fcntl.h>
#include <sys/wait.h>

#include <vector>

#include <symbolic/rules.h>
#include <symbolic/predicates.h>
#include <symbolic/state.h>
#include <symbolic/scenario.h>

#include <boost/thread.hpp>

#include <rexd/config_reader.h>
#include <ippc_wrapper/ippc_server.h>


/**
 * \class IppcPlanner
 * \brief IPPC planner wrapper. Launches and connects to a planner using the IPPC 2011 interface.
 * 
 * Inherits functionality from planner.h.\n
 * Works with glutton/gourmand planner from IPPC 2011.\n
 * TODO: make it work with PROST. \n
 * TODO: see if better planners appear in IPPC 2014.
 */
class IppcPlanner : public Planner {
public:
//     std::string rules_file_path;

    IppcPlanner(const ConfigReader& config_reader);
    ~IppcPlanner();

    using Planner::plan; // Once you overload a function from Base class in Derived class all functions with the same name in the Base class get hidden in Derived class.
    // SymbolOptional Planner::plan(RexdScenario rexd_scenario, const bool thoroughly = true, const bool verbose = false) const;
    virtual SymbolOptional plan(RexdScenario rexd_scenario, float& planning_result, const bool thoroughly = true, const bool verbose = false) const;
    virtual std::vector< Symbol > plan_n_actions(const RexdScenario& rexd_scenario, const uint n_actions, bool &reached_goal, const bool thoroughly = true, const bool verbose = false) const;

    virtual int estimate_distance_to_finish(const RexdScenario &rexd_scenario);

private:
    std::string planner_path;
    uint planner_port;

    std::string ppddl_rules_file;
    std::string ppddl_template_file;

    uint planning_miliseconds;
    uint planning_fast_miliseconds;
    double planning_epsilon;
    uint planning_horizon;
    float noop_reward;

    virtual SymbolOptional ippcPlan(const RexdScenario& rexd_scenario, const bool thoroughly, float& planning_result) const;
    void execute_glutton_planner_command(const bool thoroughly) const;
};

#endif
