/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 2013  <copyright holder> <email>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rexd/auto_teacher.h"


AutoTeacher::AutoTeacher(const ConfigReader& config_reader_):
    config_reader(config_reader_),
    complete_retries(0),
    iteration(0)
{
    goals = GoalGroup(config_reader.get_path("goal_file"));
    // create planner
    planner = PlannerFactory::get_auto_teacher_planner(config_reader);
    rules_ptr = RuleSetPtr(new RuleSet(config_reader.get_path("ground_truth_nid_rules_path")));
}

// AutoTeacher::AutoTeacher(const AutoTeacher& other)
// {
//
// }

AutoTeacher::~AutoTeacher()
{

}

float AutoTeacher::get_expected_action_success_probability(const State& state, const Symbol& action) const
{
    Scenario scenario(*rules_ptr, state, goals, config_reader);
    return planner->get_expected_action_success_probability(scenario, action);
}

SymbolOptional AutoTeacher::get_action_simple(const State& state) const
{
    Scenario scenario(*rules_ptr, state, goals, config_reader);
    return planner->plan(scenario, true, false);
}

SymbolOptional AutoTeacher::get_action(const State& state,
                                  const std::vector<Symbol>& action_list,
                                  const std::vector<std::string> actions_requiring_excuse)
{
    SymbolOptional action_opt = SymbolOptional();
    std::string action_string;
    CINFO("teacher") << "Auto Teacher!";

    uint auto_teacher_retries = 0;
    while (!action_opt) {
        action_opt = get_planned_action(state, action_list, actions_requiring_excuse);

        if (config_reader.get_variable<bool>("auto_teacher_autas_hacks")) {
            get_action_autas_hacks(state, actions_requiring_excuse, action_opt);
        }

        if ((!action_opt)) {
            if (config_reader.get_variable<bool>("auto_teacher_infinite") ) {
                infinite_autoteacher(action_opt, auto_teacher_retries);
            }
            else {
                action_opt = SymbolOptional (MAKE_SYMBOL_HELP); // read_new_action();
            }
        }
    }

    return action_opt;
}


SymbolOptional AutoTeacher::get_planned_action(const State& state,
                                     const std::vector<Symbol>& action_list,
                                     const std::vector<std::string> actions_requiring_excuse)
{
    SymbolOptional planned_action;
    Scenario scenario(*rules_ptr, state, goals, config_reader);
    RexdScenario rexd_scenario(scenario);
    rexd_scenario.set_vmin();

    SymbolOptional teacher_plan = planner->plan(rexd_scenario, true, false);
    if (teacher_plan && (*teacher_plan != MAKE_SYMBOL_TEACHER)) {
        Symbol action = *teacher_plan;
        planned_action = SymbolOptional(action);
        Symbol symbolic_action = Symbol(action);
        symbolic_action.make_symbolic();

        // always add if it is a new action
        if (config_reader.get_variable<bool>("auto_teacher_zealous")) {
            CINFO("teacher") << "Auto teacher: " << action << " - zealous mode.";
        }
        else if (std::find(action_list.begin(), action_list.end(), symbolic_action)!=action_list.end()) {
            // not a new action
            // add action only if it is in actions requiring excuse
            if (std::find(actions_requiring_excuse.begin(), actions_requiring_excuse.end(), planned_action->name)==actions_requiring_excuse.end()) {
                // action not found in actions_requiring_excuse
                CINFO("teacher") << "Auto teacher got action: " << action << " but is not recommended by excuses. Requesting human teacher help.";
                CINFO("teacher") << "Waiting for new action: ";
                planned_action = SymbolOptional();
            }
            else if (config_reader.get_variable<bool>("auto_teacher_excuse_autoexecute")) {
                CINFO("teacher") << "Action " << action << " matches excuses recommendation, executing it.";
            }
            else {
                CINFO("teacher") << "Action " << action << " matches excuses recommendation.";
                CINFO("teacher") << "Waiting for new action: ";
                planned_action = SymbolOptional();
            }
        }
        else {
            CINFO("teacher") << "Action " << action << " is a new action. Selecting it as this unknown action seems neccesary for this task.";
        }
    }
    else { // no plan or teacher action obtained (which means reward < Vmin -> no plan)
        CINFO("teacher") << "Auto teacher failed (dead end?)";
        CINFO("teacher") << "Waiting for new action: ";
        if (config_reader.get_variable<bool>("auto_teacher_autodead")) {
            planned_action = SymbolOptional(MAKE_SYMBOL_DEAD);
        }
        else {
            planned_action = SymbolOptional();
        }
    }
    return planned_action;
}


void AutoTeacher::get_action_autas_hacks(const State& state,
        const std::vector<std::string> actions_requiring_excuse,
        SymbolOptional& action_opt)
{
    if ((actions_requiring_excuse.size() > 0) && (actions_requiring_excuse.front() == "placependulum")) {
        action_opt = SymbolOptional(Symbol("placeshaft()"));
    }
    else if ((actions_requiring_excuse.size() > 0) && (actions_requiring_excuse.front() == "screwpendulumhead")) {
        if (state.has_predicate(get_predicate_from_string("shaftplaced_sh()"))) {
            action_opt = SymbolOptional(Symbol("placependulum()"));
        }
        else {
            action_opt = SymbolOptional(Symbol("placeshaft()"));
        }
    }
}


void AutoTeacher::infinite_autoteacher(SymbolOptional& action_opt, uint& auto_teacher_retries)
{
    auto_teacher_retries ++;
    if (auto_teacher_retries > config_reader.get_variable<uint>("auto_teacher_infinite_retries")) {
        complete_retries++;
        if (complete_retries < config_reader.get_variable<uint>("auto_teacher_infinite_full_retries")) {
            action_opt = SymbolOptional(MAKE_SYMBOL_RETRY);
        }
        else {
            complete_retries = 0;
            action_opt = SymbolOptional(MAKE_SYMBOL_HELP);
        }
    }
}


