/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 2013  <copyright holder> <email>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "rexd/config_reader.h"

namespace po = boost::program_options;

ConfigReader::ConfigReader()
{

}

bool ConfigReader::init(const std::string& config_path, const std::string& files_path_)
{
    files_path = files_path_ + "/";

    po::options_description options("Allowed options");
    try {
        options.add_options()
        ("help", "produce help message")

        // simulator
        ("simulator", po::value<bool>()->default_value(true), "Using simulator")
        ("simulator_port", po::value<uint>()->default_value(0), "Simulator port (0 for auto)")
        ("simulator_current_state_path", po::value<std::string>()->default_value("state_simulator.dat"), "Simulator current state path")
        ("simulator_init_state_path", po::value<std::string>()->default_value("simulated_init_state.dat"), "Simulator init state path")

        // main variables
        ("scenario", po::value<std::string>(), "Scenario")
        ("learning", po::value<bool>()->default_value(true), "Use learning")
        ("monitoring", po::value<bool>()->default_value(false), "Just monitor actions")

        // transitions
        ("transitions_backup_path", po::value<std::string>()->default_value("transitions_full.dat"), "Transitions backup path")
        ("transitions_tmp_path", po::value<std::string>()->default_value("transitions_tmp.dat"), "Transitions tmp path")

        // visualizer
        ("visualization", po::value<bool>()->default_value(false), "Enable gource visualization")
        ("visualization_gource_log_path", po::value<std::string>()->default_value("gource.log"), "Gource log path")
        ("visualization_gource_captions_log_path", po::value<std::string>()->default_value("gource_caption.log"), "Gource caption log path")
        ("visualization_avisynth_log_path", po::value<std::string>()->default_value("video_editing.avs"), "Avisynth script path")
        ("visualization_subtitle_path", po::value<std::string>()->default_value("visualization.ass"), "Visualization subtitle path")
        ("visualization_subtitle_verbose_path", po::value<std::string>()->default_value("visualization_verbose.ass"), "Visualization subtitle verbose path")
        ("visualization_enable_expected_reward", po::value<bool>()->default_value(false), "Enable visualization of expected reward (subtitle)")
        ("visualization_enable_actual_reward", po::value<bool>()->default_value(true), "Enable visualization of actual reward (subtitle)")
        ("visualization_enable_current_reward", po::value<bool>()->default_value(false), "Enable visualization of current reward (subtitle)")

        // learner
        ("pasula_alpha_pen", po::value<double>()->default_value(0.25), "Pasula algorithm alpha parameter")
        ("pasula_noise_lower_bound", po::value<double>()->default_value(1e-9), "Pasula algorithm noise lower bound parameter")
        ("pasula_noise_lower_bound_default_rule", po::value<double>()->default_value(1e-11), "Pasula algorithm noise lower bound for default rule parameter")

        // planner
        ("planner", po::value<std::string>()->default_value("prada"), "Planner used")
        ("goal_file", po::value<std::string>()->default_value("reward.dat"), "Goal file path")
        ("final_reward_action", po::value<bool>()->default_value(false), "Use final reward action")

        ("prada_config", po::value<std::string>()->default_value("config"), "Prada configuration file")
        ("prada_config_learner", po::value<std::string>()->default_value("config"), "Pasula configuration file")
        ("prada_config_teacher", po::value<std::string>()->default_value("config"), "Teacher prada configuration file")

        ("nid_rules_path", po::value<std::string>()->default_value("rules.dat"), "Current NID rules")
        ("nid_finish_rule_path", po::value<std::string>()->default_value("finish_rule.dat"), "Finish NID rule")
        ("ground_truth_nid_rules_path", po::value<std::string>()->default_value("simulated_rules.dat"), "Ground truth rules")

        ("ppddl_rules_path", po::value<std::string>()->default_value("rules.ppddl"), "Current rules in ppddl format")
        ("ppddl_rules_template_path", po::value<std::string>()->default_value("template_ppddl"), "Template to generate ppddl rules")

        ("planner_path", po::value<std::string>()->default_value("./planner"), "Planner executable path")
        ("planner_port", po::value<uint>()->default_value(0), "Planner server port (0 for auto)")
        ("planning_time", po::value<uint>()->default_value(1000), "Planning time in miliseconds")
        ("planning_fast_time", po::value<uint>()->default_value(500), "Fast planning time in miliseconds")
        ("planning_epsilon", po::value<double>()->default_value(0.05), "Planner epsilon")
        ("planning_horizon", po::value<uint>()->default_value(14), "Planner horizon")
        ("planning_noop_reward", po::value<float>()->default_value(0.1), "Planner noop reward")

        // rule analysis
        ("remove_nop_rules", po::value<bool>()->default_value(true), "Remove nop rules to speed up planning")
        ("rule_analysis", po::value<bool>()->default_value(true), "Analyze rules to explain planning failures")
        ("rule_analysis_equivalent_rules", po::value<bool>()->default_value(true), "Search for rule alternatives")
        ("rule_analysis_subgoals", po::value<bool>()->default_value(true), "Use subgoals")


        // exploration
        ("exploration", po::value<bool>()->default_value(true), "Use exploration")
        ("exploration_known_threshold", po::value<uint>()->default_value(2), "Number of experiences to consider a state-action as known")
        ("use_rmax", po::value<bool>()->default_value(false), "Use R-MAX algorithm")
        ("use_vmin", po::value<bool>()->default_value(false), "Use R-MAX with V-MIN algorithm")
        ("rmax_reward", po::value<float>()->default_value(15), "Rmax reward")
        ("vmin_reward", po::value<float>()->default_value(0.1), "Vmin reward")
        ("confirm_dangerous_actions", po::value<bool>()->default_value(false), "Ask for confirmation before executing dangerous actions.")

        // Teacher and auto-teacher
        ("teacher", po::value<bool>()->default_value(true), "Teacher available")
        ("teacher_can_fail", po::value<bool>()->default_value(true), "Teacher actions can fail.")
        ("auto_teacher", po::value<bool>()->default_value(true), "User automatic teacher")
        ("auto_teacher_excuse_autoexecute", po::value<bool>()->default_value(true), "Automatically execute recommended action when matches excuse")
        ("auto_teacher_autodead", po::value<bool>()->default_value(false), "Set as dead when autoteacher does not find a plan")
        ("auto_teacher_zealous", po::value<bool>()->default_value(false), "All autoteacher plans are valid")
        ("auto_teacher_autas_hacks", po::value<bool>()->default_value(false), "Hacks for autas")
        ("auto_teacher_infinite", po::value<bool>()->default_value(true), "Retry planning until a satisfactory solution is found")
        ("auto_teacher_infinite_retries", po::value<uint>()->default_value(5), "Number of autoteacher retries)")
        ("auto_teacher_infinite_full_retries", po::value<uint>()->default_value(3), "Number of full retries (with full planning")
        ("auto_teacher_dangerous_actions", po::value<bool>()->default_value(false), "Use autoteacher to confirm dangerous actions")

        ("auto_experiments", po::value<bool>()->default_value(true), "Start new episodes automatically")
        ("auto_experiments_num", po::value<uint>()->default_value(10), "Number of episodes per problem")
        ("auto_experiments_auto_finish", po::value<bool>()->default_value(true), "Finish after completing a problem")
        ("auto_experiments_auto_next", po::value<bool>()->default_value(false), "Continue with the following problem (if exists)")
        ("auto_experiments_auto_next_num", po::value<uint>()->default_value(1), "Number of different problems to execute")
        ("auto_experiments_increasing_vmin", po::value<bool>()->default_value(false), "Increasing Vmin")
        ("auto_experiments_increasing_vmin_step", po::value<uint>()->default_value(3), "Number of episodes before increasing Vmin")
        ("auto_experiments_increasing_vmin_num", po::value<uint>()->default_value(1), "Number of times to increase Vmin")
        ("auto_experiments_increasing_vmin_quantity", po::value<float>()->default_value(1.0), "Quantity Vmin is increased")
        
        
        ("auto_experiments_command", po::value<bool>()->default_value(false), "Request new command after completing an scenario")
        ;

        std::ifstream ifs(config_path.c_str());
        po::store(po::parse_config_file(ifs, options), vm);
//         po::store(po::parse_command_line(argc, argv, options), vm);
        po::notify(vm);

        if (vm.count("help")) {
            std::cout << options << "\n";
            exit(0);
        }
    }
    catch(std::exception& e) {
        std::cerr << "ConfigReader: " << config_path << " - ERROR: " << e.what() << "\n";
        std::cout << options << "\n";
        exit(-1);
    }
    catch(...) {
        std::cerr << "Exception of unknown type!\n";
        return false;
    }
    return true;
}

ConfigReader::ConfigReader(const ConfigReader& other):
    files_path(other.files_path)
{
    vm = other.vm;
}

ConfigReader::~ConfigReader()
{

}


