#include "rexd/learner.h"


Learner::Learner() {
}


RuleSetPtr Learner::learn_action_rules(const RuleSet& previous_rules, const TransitionGroup& transitions, const Symbol action)
{
    RuleSetPtr removed_rules, added_rules;
    return learn_action_rules(previous_rules, transitions, action, removed_rules, added_rules);
}

RuleSetPtr Learner::learn_action_rules(const RuleSet& previous_rules, const TransitionGroup& transitions, const Symbol action, RuleSetPtr& removed_rules_ptr, RuleSetPtr& added_rules_ptr)
{
    int DEBUG = 0;
    TransitionGroup transitions_action = transitions.get_action_transitions(action.name);
    if (DEBUG > 0) {
        std::cout << "Learner::learn_action_rules - " << "Starting learning..." << std::endl;
    }
    added_rules_ptr = learn_quiet(transitions_action);
    if (DEBUG > 0) {
        std::cout << "Learner::learn_action_rules - " << "Finished learning." << std::endl;
    }

    RuleSetPtr result_rules = boost::make_shared<RuleSet>(RuleSet(previous_rules));

    RuleSet removed_rules = result_rules->remove_action_rules(action.name);
    removed_rules_ptr = boost::make_shared<RuleSet>(removed_rules);
    
    // maintain same identifiers as much as possible in newer rules
    added_rules_ptr->set_identifiers(removed_rules_ptr);

    // Add extra information to new rules
    // TODO (careful, now transitions include global rewards + rule rewards)
//         float reward = transitions.get_action_reward(action.name);
//         added_rules_ptr->add_goal(Goal(reward));

    // Add new rules to current rules
    result_rules->add_rules(*added_rules_ptr);
    result_rules->add_default_rule_if_empty();

    return result_rules;
}


