/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 2013  dmartinez <email>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "rexd/planner_factory.h"

PlannerPtr PlannerFactory::get_standard_planner(const ConfigReader& config_reader)
{
    PlannerPtr planner;
    if (config_reader.get_variable<std::string>("planner") == "prada") {
        CWARNING("Planning") << "PRADA planner is deprecated. IPPC is the recommended planner now";
        std::string prada_config_file = config_reader.get_path("prada_config");
        planner = boost::make_shared<PradaPlanner>(PradaPlanner(prada_config_file));
    }
    else if (config_reader.get_variable<std::string>("planner") == "ippc") {
        planner = boost::make_shared<IppcPlanner>(IppcPlanner(config_reader));
    }
    else { // error
        std::cerr << "Planner Factory::" << "ERROR!!! UNKNOWN PLANNER TYPE " << config_reader.get_variable<std::string>("planner") << std::endl;
    }
    return planner;
}


PlannerPtr PlannerFactory::get_auto_teacher_planner(const ConfigReader& config_reader)
{
    PlannerPtr planner;
    if (config_reader.get_variable<std::string>("planner") == "prada") {
        CWARNING("Planning") << "PRADA planner is deprecated. IPPC is the recommended planner now";
        std::string prada_config_file = config_reader.get_path("prada_config_teacher");
        planner = boost::make_shared<PradaPlanner>(PradaPlanner(prada_config_file));
    }
    else if (config_reader.get_variable<std::string>("planner") == "ippc") {
        planner = boost::make_shared<IppcPlanner>(IppcPlanner(config_reader));
    }
    else { // error
        std::cerr << "Planner Factory::" << "ERROR!!! UNKNOWN PLANNER TYPE " << config_reader.get_variable<std::string>("planner") << std::endl;
    }
    return planner;
}
