/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 2014  dmartinez <email>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "rexd/visualizer.h"

// If 0, only learned rules are marked
#define SHOW_ALL_EXECUTED_RULES 1

#define AVS_SLOW_SPEED 1
#define AVS_MEDIUM_SPEED 2
#define AVS_FAST_SPEED 5
#define AVS_VERYFAST_SPEED 10

#define AVS_SLOW_SPEED_STRING "1X"
#define AVS_MEDIUM_SPEED_STRING "2X"
#define AVS_FAST_SPEED_STRING "5X"
#define AVS_VERYFAST_SPEED_STRING "10X"

Visualizer::Visualizer(const ConfigReader& config_reader)
{
    init(config_reader);
}

Visualizer::~Visualizer()
{
    if (enabled) {
        end_avisynth_script(avisynth_script_file);
    }
}

void Visualizer::init(const ConfigReader& config_reader)
{
    previous_episode_number = 1;
    previous_problem_number = 1;
    previous_episode_reward = 0;
    previous_episode_start_time = 1;

    enabled = config_reader.get_variable<bool>("visualization");
    if (enabled) {
        enabled_expected_reward = config_reader.get_variable<bool>("visualization_enable_expected_reward");
        enabled_actual_reward = config_reader.get_variable<bool>("visualization_enable_actual_reward");
        enabled_current_reward = config_reader.get_variable<bool>("visualization_enable_current_reward");
        gource_log_file.open(config_reader.get_path("visualization_gource_log_path").c_str(), std::ios::app);
        gource_caption_log_file.open(config_reader.get_path("visualization_gource_captions_log_path").c_str(), std::ios::app);
        avisynth_script_file.open(config_reader.get_path("visualization_avisynth_log_path").c_str(), std::ios::app);
        subtitle_log_file.open(config_reader.get_path("visualization_subtitle_path").c_str(), std::ios::app);
        subtitle_verbose_log_file.open(config_reader.get_path("visualization_subtitle_verbose_path").c_str(), std::ios::app);
        init_subtitle(subtitle_log_file);
        init_subtitle(subtitle_verbose_log_file);
        init_avisynth_script(avisynth_script_file);

        // as no new command is triggered in the beggining, make the first log manually:
        // write_episode_log(config_reader.get_variable<std::string>("scenario"), 1, 1, 0, 1);
    }
}

void Visualizer::init_subtitle(std::ostream& subtitle_stream)
{
    subtitle_stream << "[Script Info]" << std::endl
                    << "; Script generated by REX-D" << std::endl
                    << "; http://www.iri.upc.edu/groups/perception/#ALOMS" << std::endl
                    << "Title: Default REX-D file" << std::endl
                    << "ScriptType: v4.00+" << std::endl
                    << "WrapStyle: 0" << std::endl
                    << "PlayResX: 640" << std::endl
                    << "PlayResY: 480" << std::endl
                    << "ScaledBorderAndShadow: yes" << std::endl
                    << "Video Aspect Ratio: 0" << std::endl
                    << "Video Zoom: 6" << std::endl
                    << "Video Position: 0" << std::endl
                    << "Last Style Storage: Default" << std::endl
                    << "" << std::endl
                    << "[V4+ Styles]" << std::endl
                    << "Format: Name, Fontname, Fontsize, PrimaryColour, SecondaryColour, OutlineColour, BackColour, Bold, Italic, Underline, StrikeOut, ScaleX, ScaleY, Spacing, Angle, BorderStyle, Outline, Shadow, Alignment, MarginL, MarginR, MarginV, Encoding" << std::endl
                    << "Style: Default,Arial,25,&H00FFFFFF,&H000000FF,&H00000000,&H00000000,0,0,0,0,100,100,0,0,1,2,1,2,10,10,10,1" << std::endl
                    << "Style: TitleStyle,DejaVu Sans,24,&H00FFFFFF,&H000000FF,&H00000000,&H00000000,-1,0,0,0,100,100,0,0,1,0,2,8,10,10,20,1" << std::endl
                    << "Style: RewardStyle,DejaVu Sans,20,&H00FFFFFF,&H000000FF,&H00000000,&H00000000,0,0,0,0,100,100,0,0,1,0,2,7,275,10,55,1" << std::endl
                    << "Style: CurrentRewardStyle,DejaVu Sans,20,&H00FFFFFF,&H000000FF,&H00000000,&H00000000,0,0,0,0,100,100,0,0,1,0,2,7,225,10,80,1" << std::endl
                    << "" << std::endl
                    << "[Events]" << std::endl
                    << "Format: Layer, Start, End, Style, Name, MarginL, MarginR, MarginV, Effect, Text" << std::endl
                    << "Comment: 0,0:00:00.00,0:00:00.00,Default,,0000,0000,0000,,This line is a comment." << std::endl;
//     << "Dialogue: 0,0:00:00.00,0:00:02.00,Default,,0000,0000,0000,,\"Loading rewards...\"" << std::endl;
}

void Visualizer::init_avisynth_script(std::ostream& avisynth_stream)
{
    avisynth_stream << "# avxsynth script" << std::endl
                    << "full_video = FFVideoSource(\"video_realtime.mp4\")" << std::endl
                    << "video_fps = FrameRate(full_video)" << std::endl
                    << "video_width = Width(full_video)" << std::endl
                    << "video_height = Height(full_video)" << std::endl
                    << "speed_multiplier=1" << std::endl
                    << std::endl;
}

void Visualizer::end_avisynth_script(std::ostream& avisynth_stream)
{
    avisynth_stream << std::endl
                    << avisynth_files_to_concat << std::endl
                    << "return final_video" << std::endl
                    << std::endl;
}

void Visualizer::write_episode_log(const std::string& scenario, const ulong problem_number, const ulong episode_number, const float reward, const ulong total_number_iterations)
{
    if (!enabled)
        return;

    gource_caption_log_file << total_number_iterations << "|" << scenario << " " << problem_number << " - " << episode_number << std::endl;

    subtitle_log_file << "Dialogue: 0," << utils::get_parsed_time_from_seconds(previous_episode_start_time) << ".00,"
                      << utils::get_parsed_time_from_seconds(total_number_iterations) << ".00,TitleStyle,,0000,0000,0000,,"
                      << "{\\fad(1200,250)}Scenario: " << scenario << " " << previous_problem_number << " - " << previous_episode_number << std::endl;

    std::stringstream avs_speed;
    std::string avs_speed_text;
    std::stringstream video_part_name_ss;
    video_part_name_ss <<  "video_part" << previous_problem_number << "_" << previous_episode_number;
    if (previous_episode_number <= 2) {
        avs_speed << ".ChangeFPS(speed_multiplier*video_fps/" << AVS_SLOW_SPEED << ").AssumeFPS(video_fps)";
        avs_speed_text = AVS_SLOW_SPEED_STRING;
    }
    else if (previous_episode_number <= 4) {
        avs_speed << ".ChangeFPS(speed_multiplier*video_fps/" << AVS_MEDIUM_SPEED << ").AssumeFPS(video_fps)";
        avs_speed_text = AVS_MEDIUM_SPEED_STRING;
    }
    else if (previous_episode_number <= 8) {
        avs_speed << ".ChangeFPS(speed_multiplier*video_fps/" << AVS_FAST_SPEED << ").AssumeFPS(video_fps)";
        avs_speed_text = AVS_FAST_SPEED_STRING;
    }
    else {
        // HACK because min fps are ~6
        avs_speed << ".ChangeFPS(speed_multiplier*video_fps/" << AVS_VERYFAST_SPEED/2 << ").AssumeFPS(video_fps)" << ".ChangeFPS(video_fps/" << 2 << ").AssumeFPS(video_fps)";
        avs_speed_text = AVS_VERYFAST_SPEED_STRING;
    }

    avisynth_script_file << video_part_name_ss.str() << "="
                         << "Trim(full_video, " << previous_episode_start_time << "*60, " << total_number_iterations << "*60)"
                         << avs_speed.str() << std::endl
                         << video_part_name_ss.str() << "=" << video_part_name_ss.str() << "."
                         << "Subtitle(string (\"" << avs_speed_text << "\"), text_color=$ffffff, x=video_width-140, y=video_height-70, font =\"Arial\", size=30, "
                         << "first_frame =0, last_frame = FrameCount(" << video_part_name_ss.str() << "), lsp=30, align=7, spc =62)" << std::endl;

    if (avisynth_files_to_concat.size()==0) { // initial
        avisynth_files_to_concat = "final_video=" + video_part_name_ss.str();
    }
    else {
        avisynth_files_to_concat += "++" + video_part_name_ss.str();
    }

    if (enabled_actual_reward) {
        subtitle_log_file << "Dialogue: 0," << utils::get_parsed_time_from_seconds(previous_episode_start_time) << ".00,"
                          << utils::get_parsed_time_from_seconds(total_number_iterations) << ".00,RewardStyle,,0000,0000,0000,,"
                          << "Reward: {\\b1}" << reward << "{\\b0}" << std::endl;
    }

    previous_episode_number = episode_number;
    previous_problem_number = problem_number;
    previous_episode_reward = reward;
    previous_episode_start_time = total_number_iterations;
}

void Visualizer::write_status_log(const float accumulated_reward, const float expected_reward, const ulong total_number_iterations)
{
    if (!enabled)
        return;

    if (enabled_expected_reward) {
        subtitle_verbose_log_file << "Dialogue: 0," << utils::get_parsed_time_from_seconds(total_number_iterations) << ".00,"
                                  << utils::get_parsed_time_from_seconds(total_number_iterations+1) << ".00,CurrentRewardStyle,,0000,0000,0000,,"
                                  << "Expected reward: {\\b1}" << expected_reward << "{\\b0}" << std::endl;
    }
    if (enabled_current_reward) {
        subtitle_verbose_log_file << "Dialogue: 0," << utils::get_parsed_time_from_seconds(total_number_iterations) << ".00,"
                                  << utils::get_parsed_time_from_seconds(total_number_iterations+1) << ".00,CurrentRewardStyle,,0000,0000,0000,,"
                                  << "Current reward:    {\\b1}" << accumulated_reward << "{\\b0}" << std::endl;
    }
}

void Visualizer::write_learning_log(const RuleSet& removed_rules, const RuleSet& added_rules, const std::string& context, const ulong total_number_iterations)
{
    if (!enabled)
        return;
    // learning is done in the iteration after execution, therefore we show here total_number_iterations-1 for consistency reasons

    std::vector<int> preconditions_diff, outcomes_num_diff;
    std::vector<double> first_outcome_prob_diff;
    RuleSet new_rules, old_rules, new_changed_rules, old_changed_rules;
    removed_rules.analyze_differences(added_rules, new_rules, old_rules, old_changed_rules, new_changed_rules, preconditions_diff, outcomes_num_diff, first_outcome_prob_diff);

    BOOST_FOREACH( RulePtr rule, new_rules) {
        write_learning_log_rule(rule, context, "A", total_number_iterations - 1);
//         LOG(DEBUG) << "Adding Gource logs. - Add action";
    }
    BOOST_FOREACH( RulePtr rule, old_rules) {
        write_learning_log_rule(rule, context, "D", total_number_iterations - 1);
//         LOG(DEBUG) << "Adding Gource logs. - Remove action";
    }
    RuleSet::const_iterator old_changed_rule_it = old_changed_rules.begin();
    BOOST_FOREACH( RulePtr rule, new_changed_rules) {
        if ((SHOW_ALL_EXECUTED_RULES) || (*rule != **old_changed_rule_it)) {
            write_learning_log_rule(rule, context, "M", total_number_iterations - 1);
//             LOG(DEBUG) << "Adding Gource logs. - Change action";
        }
        ++old_changed_rule_it;
    }
    /*
        LOG(INFO) << "GOURCE INFORMATION!!!";
        LOG(INFO) << "ADDED RULES " << added_rules;
        LOG(INFO) << "REMOVED RULES " << removed_rules;
        LOG(INFO) << "new_rules " << new_rules;
        LOG(INFO) << "old_rules " << old_rules;
        LOG(INFO) << "new_changed_rules " << new_changed_rules;
        LOG(INFO) << "old_changed_rules " << old_changed_rules;
        LOG(INFO) << "preconditions_diff " << preconditions_diff;
        LOG(INFO) << "outcomes_num_diff " << outcomes_num_diff;
        LOG(INFO) << "first_outcome_prob_diff " << first_outcome_prob_diff;

        read_new_action();*/
}

void Visualizer::write_learning_log_rule(RuleConstPtr rule_ptr, const std::string& context, const std::string& type, const ulong total_number_iterations)
{
    if (!enabled)
        return;
//     if (!rule_ptr->has_useful_outcome()) { // hide rules without useful outcome
//         return;
//     }
//     gource_log_file << std::time(0);
    gource_log_file << total_number_iterations;
    gource_log_file << "|";
    gource_log_file << context;
    gource_log_file << "|";
    gource_log_file << type;
    gource_log_file << "|";
    gource_log_file << rule_ptr->name << "/" << rule_ptr->identifier;
    gource_log_file << "|";
    // Optional
    if (!rule_ptr->has_useful_outcome()) { // hide rules without useful outcome
        gource_log_file << "000000";
    }
    else if (rule_ptr->outcomes.front().probability < 1.0) {
        int hex_probability_representation = round(rule_ptr->outcomes.front().probability * 12); // 12 instead of 16 so it is more clear to see
        std::stringstream ss;
        ss << std::hex << hex_probability_representation;
        gource_log_file << ss.str() << ss.str() << ss.str() << ss.str() << ss.str() << ss.str(); // xD
    }
    else {
        gource_log_file << "FFFFFF";
    }
    gource_log_file << "" << std::endl;
}

