#include <symbolic/predicates.h>

using namespace std;

// Operators
bool operator==(const Predicate &p1, const Predicate &p2)
{
    return ((*static_cast<const Symbol*>(&p1) == p2) && (p1.positive == p2.positive));
}

bool Predicate::symbolic_compare(const Predicate& p) const
{
    return ((static_cast<const Symbol*>(&*this)->symbolic_compare(p)) && (positive == p.positive));
}

bool operator==(const PredicateGroup &p1, const PredicateGroup &p2)
{
    bool result = true;
    PredicateGroup p2_copy(p2);
    for (PredicateGroup::const_iterator it_pred = p1.begin() ; it_pred != p1.end(); it_pred++) {
        bool present = false;
        for (PredicateGroup::iterator it_pred2 = p2_copy.begin() ; it_pred2 != p2_copy.end();) {
            if (*it_pred == *it_pred2) {
                present = true;
                it_pred2 = p2_copy.erase(it_pred2);
            } else {
                ++it_pred2;
            }
        }
        if (!present) {
            result = false;
            break;
        }
    }
    if (p2_copy.size() != 0) { // all predicates should have been removed
        result = false;
    }
    return result;
}

bool operator==(const Outcome &o1, const Outcome &o2)
{
    return ((*static_cast<const PredicateGroup*>(&o1) == o2) && (o1.probability == o2.probability));
}

bool operator==(const OutcomeList &o1, const OutcomeList &o2)
{
    bool result = true;
    OutcomeList o2_copy(o2);
    for (OutcomeList::const_iterator out_it = o1.begin() ; out_it != o1.end(); out_it++) {
        bool present = false;
        for (OutcomeList::iterator out_it2 = o2_copy.begin() ; out_it2 != o2_copy.end();) {
            if (*out_it == *out_it2) {
                present = true;
                out_it2 = o2_copy.erase(out_it2);
            } else {
                ++out_it2;
            }
        }
        if (!present) {
            result = false;
            break;
        }
    }
    if (o2_copy.size() != 0) { // all outcomes should have been removed
        result = false;
    }
    return result;
}

std::ostream& operator<<(std::ostream &out, const Predicate& p)
{
    if (not p.positive)
        out << "-";
    out << *static_cast<const Symbol*>(&p);
    return out;
}

std::ostream& operator<<(std::ostream &out, const PredicateGroup& p)
{
    for (size_t i = 0; i < p.size(); i++) {
        out << p[i];
        if ((i + 1) < p.size())
            out << " ";
    }
    return out;
}

std::ostream& operator<<(std::ostream &out, const Outcome& o)
{
    out << o.probability << " ";
    if (o.noisy) {
        out << "<noise>";
    } else {
        out << *static_cast<const PredicateGroup*>(&o);
    }
    return out;
}

std::ostream& operator<<(std::ostream &out, const Goal& g)
{
    out << g.reward << " " << *static_cast<const PredicateGroup*>(&g) << std::flush;
    return out;
}

std::ostream& operator<<(std::ostream &out, const GoalGroup& g)
{
    for (GoalGroup::const_iterator goal_it = g.begin(); goal_it != g.end(); ++goal_it) {
        out << *goal_it << std::endl;
    }
    return out;
}

bool operator<(const Predicate& l, const Predicate& r)
{
    if (*static_cast<const Symbol*>(&l) < *static_cast<const Symbol*>(&r))
        return true;
    else if (*static_cast<const Symbol*>(&r) < *static_cast<const Symbol*>(&l))
        return false;
    else if (l.positive < r.positive)
        return true;
    else
        return false;
}

bool operator<(const PredicateGroup& l, const PredicateGroup& r)
{
    if (l.size() < r.size())
        return true;
    else if (r.size() < l.size())
        return false;

    PredicateGroup l_copy(l);
    PredicateGroup r_copy(r);
    std::sort(l_copy.begin(), l_copy.end());
    std::sort(r_copy.begin(), r_copy.end());

    PredicateGroup::iterator l_it = l_copy.begin();
    for (PredicateGroup::iterator r_it = r_copy.begin(); r_it != r_copy.end(); ++r_it) {
        if (*l_it < *r_it)
            return true;
        else if (*r_it < *l_it)
            return false;
        ++l_it;
    }
    return false;
}

bool operator<(const Outcome& l, const Outcome& r)
{
    if (*static_cast<const PredicateGroup*>(&l) < *static_cast<const PredicateGroup*>(&r))
        return true;
    else if (*static_cast<const PredicateGroup*>(&r) < *static_cast<const PredicateGroup*>(&l))
        return false;
    else if (l.probability < r.probability)
        return true;
    else if (r.probability < l.probability)
        return false;
    else
        return false;
}

bool operator<(const OutcomeGroup& l, const OutcomeGroup& r)
{
    if (l.size() < r.size())
        return true;
    else if (r.size() < l.size())
        return false;

    OutcomeGroup l_copy(l);
    OutcomeGroup r_copy(r);
    std::sort(l_copy.begin(), l_copy.end());
    std::sort(r_copy.begin(), r_copy.end());

    OutcomeGroup::iterator l_it = l_copy.begin();
    for (OutcomeGroup::iterator r_it = r_copy.begin(); r_it != r_copy.end(); ++r_it) {
        if (*l_it < *r_it)
            return true;
        else if (*r_it < *l_it)
            return false;
        ++l_it;
    }
    return false;
}

void Predicate::write_PPDDL(std::ostream &out) const
{
    if (!positive) {
        out << "(not ";
    }
    (static_cast<const Symbol*>(this))->write_PPDDL(out);
    if (!positive) {
        out << ")";
    }
}

void PredicateGroup::write_PPDDL(std::ostream &out) const
{
    // Maximun of 15 predicates per line (to not exceed character limit in editors)
    uint number_predicates = 0;
    out << "(and ";
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); ++it_pred) {
        it_pred->write_PPDDL(out);
        out << " ";
        if (number_predicates++ > 15) {
            number_predicates = 0;
            out << std::endl;
        }
    }
    out << ")";
}

void PredicateGroup::write_negated_PPDDL(std::ostream &out) const
{
    // Maximun of 15 predicates per line (to not exceed character limit in editors)
    uint number_predicates = 0;
    out << "(or ";
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); ++it_pred) {
        it_pred->get_negated().write_PPDDL(out);
        out << " ";
        if (number_predicates++ > 15) {
            number_predicates = 0;
            out << std::endl;
        }
    }
    out << ")";
}

void PredicateGroup::PPDDL_grounding()
{
    for (PredicateGroup::iterator it_pred = begin() ; it_pred != end(); ++it_pred) {
        it_pred->PPDDL_grounding();
    }
}


void Outcome::write_PPDDL(std::ostream &out) const
{
    out << " " << probability << " ";
    (static_cast<const PredicateGroup*>(this))->write_PPDDL(out);
}


void Goal::write_PPDDL(std::ostream &out) const
{
    if (size() > 0) {
        out << "    (when  ";
        (static_cast<const PredicateGroup*>(this))->write_PPDDL(out);
    }
    out << " (";
    if (reward >= 0)
        out << "increase ";
    else
        out << "decrease ";
    out << "(reward) ";
    out << std::abs(reward);
    out << ")";
    if (size() > 0) {
        out << ")";
    }
    out << std::endl;
}

void GoalGroup::write_PPDDL(std::ostream &out) const
{
    out << std::endl << "    ; Reward" << std::endl;
    for (GoalGroup::const_iterator goal_it = begin(); goal_it != end(); ++goal_it) {
        goal_it->write_PPDDL(out);
    }
}



// Predicate
std::vector<std::string> Predicate::get_symbolic_params() const
{
    std::vector<std::string> symbolic_params;
    for (std::vector<std::string>::const_iterator param_it = param_names.begin(); param_it != param_names.end(); ++param_it) {
        if (utils::belongs_to_vector<std::string>(*param_it, DEFAULT_PARAM_NAMES_VECTOR)) {
            symbolic_params.push_back(*param_it);
        }
    }
    return symbolic_params;
}


std::vector<Predicate> Predicate::get_symbolic_predicates_permutations(const std::vector<std::string> symbolic_params) const
{
    PredicateGroup grounded_predicates;
    vector<string> full_symbolic_params(symbolic_params);
    while (full_symbolic_params.size() < param_names.size()) { // add default parameters if there are not enough of them
        full_symbolic_params.push_back(DEFAULT_PARAM_NAMES[param_names.size()]);
    }
    std::sort(full_symbolic_params.begin(), full_symbolic_params.end());

    do {
        Predicate predicate(*this);
        for (size_t j = 0; j < param_names.size(); j++) {
            predicate.param_names[j] = full_symbolic_params[j];
        }
        grounded_predicates.push_back(predicate);
    } while (std::next_permutation(full_symbolic_params.begin(), full_symbolic_params.end()));

    grounded_predicates.remove_duplicated();
    return grounded_predicates;
}

/**
 * ***************
 * PredicateGroup
 */

bool PredicateGroup::satisfies(const Predicate predicate) const
{
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        if (predicate.symbolic_compare(*it_pred)) {
            return true;
        }
    }
    return false;
}

// PredicateGroup
PredicateGroup::PredicateGroup(const PredicateGroup& pred_group) :
    std::vector<Predicate>(pred_group)
{ }

PredicateGroup::PredicateGroup(const PredicateList& predicates) :
    std::vector<Predicate>(predicates)
{ }

PredicateGroup::PredicateGroup(std::string predicates_)
{
    std::string item, aux, line;
    bool positive;

    // remove leading and trailing spaces
    line = utils::trim(predicates_);

    std::stringstream ss(line);

    // case of empty string
    if ((line == "") || (line == "--")) {
        return;
    }

    while (std::getline(ss, item, ')')) {
        item = utils::trim(item);
//         std::cout << "Generating predicate for: " << item << std::endl;

        if (item.size() > 1) { // if something remains
            // add ) to the end to complete it
            item = item + ')';

            // if begins with , remove it
            aux = item.substr(0, 1);
            if (aux.compare(",") == 0)
                item = item.substr(1);

            item = utils::trim(item);
            aux = item.substr(0, 1);
            if (aux.compare("-") == 0) {
                item = item.substr(1);
                positive = false;
            } else
                positive = true;

            push_back(Predicate(Symbol(item), positive));
        }
    }
}

bool PredicateGroup::is_satisfied(const PredicateGroup current_predicates) const
{
    bool satisfied = true;
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        bool pred_satisfied = false;
        bool negated_pred_present = false;
        for (PredicateGroup::const_iterator it_pred2 = current_predicates.begin() ; it_pred2 != current_predicates.end(); it_pred2++) {
            if ((*it_pred) == (*it_pred2)) {
                pred_satisfied = true;
            }
            if ((*it_pred) == (it_pred2->get_negated())) {
                negated_pred_present = true;
            }
        }
        // negated predicates may not be present
        if ((!it_pred->positive) && (!negated_pred_present)) {
            pred_satisfied = true;
        }
        if (!pred_satisfied) {
            satisfied = false;
            return false; // ugly but quicker
        }
    }
//     std::cout << "Checking predicates\nPreconditions:" << *this << "\nState:" << PredicateGroup(current_predicates) << "\nResult: " << satisfied << std::endl;
    return satisfied;
}

int PredicateGroup::count_satisfied(const PredicateGroup current_predicates) const
{
    int n_satisfied = 0;
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        bool pred_satisfied = false;
        for (PredicateGroup::const_iterator it_pred2 = current_predicates.begin() ; it_pred2 != current_predicates.end(); it_pred2++) {
            if (*it_pred == *it_pred2) {
                pred_satisfied = true;
            }
        }
        if (pred_satisfied) {
            n_satisfied++;
        }
    }
//         std::cout << "Counting predicates\nPreconditions:" << *this << "\nState:" << PredicateGroup(current_predicates) << "\nResult: " << n_satisfied << std::endl;
    return n_satisfied;
}

bool PredicateGroup::has_predicate(const Predicate& predicate) const
{
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        if (it_pred->symbolic_compare(predicate)) {
            return true;
        }
    }
    return false;
}

int PredicateGroup::number_parameters() const
{
    std::set<std::string> parameters;

    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        for (std::vector<std::string>::const_iterator it_param = it_pred->param_names.begin(); it_param != it_pred->param_names.end(); it_param++) {
            if (parameters.count(*it_param) == 0)
                parameters.insert(*it_param);
        }
    }

    return parameters.size();
}

void PredicateGroup::change_variable(const std::string& orig_var, const std::string& new_var)
{
    for (PredicateGroup::iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        for (std::vector<std::string>::iterator it_param = it_pred->param_names.begin(); it_param != it_pred->param_names.end(); it_param++) {
            if (it_param->compare(orig_var) == 0) {
                *it_param = new_var;
            }
        }
        it_pred->autodetect_symbolic();
    }
}

void PredicateGroup::update_predicate(const Predicate& new_predicate)
{
    bool is_present = false;
    for (PredicateGroup::iterator it_pred = begin() ; it_pred != end(); ++it_pred) {
        if (*static_cast<const Symbol*>(&new_predicate) == *it_pred) { // if equal name and param names
            it_pred->positive = new_predicate.positive;
            is_present = true;
        }
    }
    if (!is_present) {
        push_back(new_predicate);
    }
}

void PredicateGroup::update_predicates(const PredicateGroup& new_predicates)
{
    BOOST_FOREACH(const Predicate & predicate, new_predicates) {
        update_predicate(predicate);
    }
}

std::vector<std::string> PredicateGroup::get_variables() const
{
    std::vector<std::string> result;
    std::set<std::string> variables;
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        for (std::vector<std::string>::const_iterator it_param = it_pred->param_names.begin(); it_param != it_pred->param_names.end(); it_param++) {
            variables.insert(*it_param);
        }
    }
    result.assign(variables.begin(), variables.end());
    return result;
}

std::vector<int> PredicateGroup::get_numeric_variables() const
{
    std::vector<int> result;
    std::vector<std::string> variables = get_variables();
    for (std::vector<std::string>::iterator var_it = variables.begin() ; var_it != variables.end(); var_it++) {
        result.push_back(atoi(var_it->c_str()));
    }
    return result;
}

PredicateGroup PredicateGroup::get_predicates_with_var(const std::string variable) const
{
    std::vector<std::string> variables;
    variables.push_back(variable);
    return get_predicates_with_vars(variables);
}

PredicateGroup PredicateGroup::get_predicates_with_vars(const std::vector< std::string > variables) const
{
    PredicateGroup preds_with_vars;
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        for (std::vector<std::string>::const_iterator it_param = it_pred->param_names.begin(); it_param != it_pred->param_names.end(); it_param++) {
            for (std::vector<std::string>::const_iterator it_var = variables.begin(); it_var != variables.end(); it_var++) {
                if (it_param->compare(*it_var) == 0) {
                    preds_with_vars.push_back(*it_pred);
                }
            }
        }
    }

    PredicateGroup result(preds_with_vars);
    result.remove_duplicated();
    return result;
}

void PredicateGroup::add_positive_differences(PredicateGroup compared_group)
{
    PredicateGroup differences = compared_group.get_differences(*this);
    for (PredicateGroup::iterator it_pred = differences.begin() ; it_pred != differences.end(); it_pred++) {
        if (it_pred->positive) {
            this->push_back(*it_pred);
        }
    }
//         this->insert( this->end(), differences.begin(), differences.end() );
    remove_duplicated();
}

PredicateGroup PredicateGroup::get_differences(const PredicateGroup compared_group) const
{
    PredicateGroup compared_preds(compared_group);
    PredicateGroup original_preds(*this);
    PredicateGroup differences;
    bool is_present;

    original_preds.remove_duplicated();
    for (PredicateGroup::const_iterator it_pred = original_preds.begin() ; it_pred != original_preds.end(); it_pred++) {
        is_present = false;
        for (PredicateGroup::iterator it_pred2 = compared_preds.begin() ; it_pred2 != compared_preds.end();) {
            if (*it_pred == *it_pred2) {
                is_present = true;
                it_pred2 = compared_preds.erase(it_pred2);
            } else {
                ++it_pred2;
            }
        }
        if (!is_present) {
            differences.push_back(it_pred->get_negated());
        }
    }

    differences.insert(differences.end(), compared_preds.begin(), compared_preds.end());

    differences.remove_duplicated();
    //std::cout << "Differences: " << differences << std::endl;
    return differences;
}

void PredicateGroup::remove(const PredicateGroup predicates_to_remove)
{
    for (PredicateGroup::const_iterator it_pred = predicates_to_remove.begin() ; it_pred != predicates_to_remove.end(); ++it_pred) {
        remove(*it_pred);
    }
}

void PredicateGroup::remove(const Predicate predicate)
{
    for (PredicateGroup::iterator it_pred = begin() ; it_pred != end();) {
        if (predicate.symbolic_compare(*it_pred)) {
            it_pred = erase(it_pred);
        } else {
            ++it_pred;
        }
    }
}

void PredicateGroup::remove_duplicated()
{
    for (PredicateGroup::iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        for (PredicateGroup::iterator it_pred2 = (it_pred + 1) ; it_pred2 != end();) {
            if (*it_pred == *it_pred2) {
                it_pred2 = erase(it_pred2);
            } else {
                ++it_pred2;
            }
        }
    }
}

void PredicateGroup::remove_duplicated_symbolic()
{
    for (PredicateGroup::iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        for (PredicateGroup::iterator it_pred2 = (it_pred + 1) ; it_pred2 != end();) {
            if (it_pred->symbolic_compare(*it_pred2)) {
                it_pred2 = erase(it_pred2);
            } else {
                ++it_pred2;
            }
        }
    }
}

void PredicateGroup::make_symbolic()
{
    for (PredicateGroup::iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        it_pred->make_symbolic();
    }
}

PredicateGroup PredicateGroup::get_negatet_predicates() const
{
    PredicateGroup result(*this);
    for (PredicateGroup::iterator it_pred = result.begin() ; it_pred != result.end(); ++it_pred) {
        it_pred->negate();
    }
    return PredicateGroup(result);
}

PredicateGroup PredicateGroup::get_symbolic_predicates() const
{
    PredicateGroup result;
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); ++it_pred) {
        if (it_pred->is_symbolic) {
            result.push_back(*it_pred);
        }
    }
    return result;
}

PredicateGroup PredicateGroup::get_symbolic_predicates_permutations(const std::vector< std::string >& param_names) const
{
    PredicateGroup result;
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); ++it_pred) {
        result.add_predicates(it_pred->get_symbolic_predicates_permutations(param_names));
    }
    return result;
}

std::vector<std::string> PredicateGroup::get_symbolic_params() const
{
    std::vector<std::string> result;
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); ++it_pred) {
        std::vector<std::string> new_symbolic_params = it_pred->get_symbolic_params();
        result.insert(result.end(), new_symbolic_params.begin(), new_symbolic_params.end());
    }
    utils::remove_duplicates<std::string>(result);
    return result;
}


void PredicateGroup::remove_negated_predicates()
{
    for (PredicateGroup::iterator it_pred = begin() ; it_pred != end();) {
        if (!it_pred->positive) {
            it_pred = erase(it_pred);
        } else {
            ++it_pred;
        }
    }
}


void PredicateGroup::remove_variables_not_present(std::vector<std::string> variables)
{
    for (PredicateGroup::iterator it_pred = begin() ; it_pred != end();) {
        if (it_pred->has_param_with_all_names(variables)) {
            ++it_pred;
        } else {
            it_pred = erase(it_pred);
        }
    }
}


void PredicateGroup::write_to_file(const std::string file_path) const
{
    int DEBUG = 0;
    std::ofstream predicates_file(file_path.c_str());
    std::cout << "PredicateGroup::" << "write_to_file: " << "Writting predicates to file " << file_path << std::endl;
    if (DEBUG)
        std::cout << "PredicateGroup::" << "write_to_file: " << "Predicates are " << *this << std::endl << std::endl;
    if (predicates_file.is_open()) {
        predicates_file << *this;
    } else
        std::cerr << "PredicateGroup::" << "write_to_file: " << "Couldn't write predicates to " << file_path << std::endl;
    predicates_file.close();
}

PredicateGroup PredicateGroup::get_not_satisfied(const PredicateGroup predicates) const
{
    PredicateGroup predicates_not_satisfied;
    for (PredicateGroup::const_iterator it_pred = begin() ; it_pred != end(); it_pred++) {
        bool pred_satisfied = false;
        for (PredicateGroup::const_iterator it_pred2 = predicates.begin() ; it_pred2 != predicates.end(); ++it_pred2) {
            if (*it_pred == *it_pred2) {
                pred_satisfied = true;
            }
        }
        if (!pred_satisfied) {
            predicates_not_satisfied.push_back(*it_pred);
        }
    }
    return predicates_not_satisfied;
}


/**
 * Outcome
 */
double Outcome::probability_obtain_predicate(const Predicate predicate) const
{
    if (satisfies(predicate))
        return probability;
    else
        return 0.0;
}


// Other


Predicate get_predicate_from_string(std::string item)
{
    std::string aux;
    bool positive;

    item = utils::trim(item);
//     std::cout << "Generating predicate for: " << item << std::endl;

    aux = item.substr(0, 1);
    if (aux.compare("-") == 0) {
        item = item.substr(1);
        positive = false;
    } else
        positive = true;

    return Predicate(Symbol(item), positive);
}



Outcome::Outcome(std::string line)
{
    std::string item;
    noisy = false;

    // remove leading spaces
    line = line.substr(line.find_first_not_of(" \t"));
    std::stringstream ss(line);

    if (std::getline(ss, item, ' ')) {
        //std::cout << "Probability: " << item << std::endl;
    }
    probability = atof(item.c_str());

    if (ss.eof())
        return;

    // get rest of the line
    std::getline(ss, item);
    if (item.find("<noise>") != std::string::npos) {
        noisy = true;
    } else if (item.find("<no-change>") != std::string::npos) {

    } else {
        replace_predicates(PredicateGroup(item));
    }
}

bool Outcome::changes_predicates() const
{
    if ((size() > 0) && (!noisy)) {
        return true;
    } else {
        return false;
    }
}

bool Outcome::is_useful() const
{
    if (changes_predicates() && (probability > 0.0)) {
        return true;
    } else {
        return false;
    }
}

bool Outcome::apply(PredicateGroup& input_predicates) const
{
    if (changes_predicates()) {
        input_predicates.update_predicates(*this);
        return true;
    } else
        return false;
}

bool OutcomeGroup::changes_predicates() const
{
    bool result = false;
    for (OutcomeGroup::const_iterator outcome_it = this->begin(); outcome_it != this->end(); outcome_it++) {
        if (outcome_it->changes_predicates()) {
            result = true;
        }
    }
    return result;
}



// Goal
Goal::Goal(const std::string& goal_string)
{
    reward = 0.0;
    std::string ss_item;
    std::stringstream goal_ss(goal_string);
    std::getline(goal_ss, ss_item, ' ');
    try {
        reward = boost::lexical_cast<float>(ss_item);
        std::getline(goal_ss, ss_item);
        if (goal_ss.str() == ss_item) { // if no predicates getline does not obtain anything new
            ss_item = "";
        }
        add_predicates(PredicateGroup(ss_item));
    } catch (boost::bad_lexical_cast &) {
        LOG(ERROR) << "Error parsing goal " << goal_string;
        LOG(WARNING) << "Compatibility mode, setting reward to 1.0 and goal predicates to " << goal_string;
        reward = 1.0;
        add_predicates(PredicateGroup(goal_string));
    }
}

float Goal::obtain_reward(const PredicateGroup& predicates) const
{
    float obtained_reward = 0;
    if (is_satisfied(predicates)) {
        obtained_reward += this->reward;
    }
    return obtained_reward;
}

// GoalGroup
GoalGroup::GoalGroup(std::string file_path)
{
    std::ifstream goals_file(file_path.c_str());
    if (goals_file.is_open()) {
        while (goals_file.good()) {
            std::string goal_string;
            std::getline(goals_file, goal_string);
            goal_string = utils::trim(goal_string);
            if (goal_string != "") {
                push_back(Goal(goal_string));
            }
        }
        goals_file.close();
    } else LOG(ERROR) << "Unable to open file: "  << file_path;
}

bool GoalGroup::is_goal(const PredicateGroup& state) const
{
    bool result = true;
    if (!state.has_predicate(Predicate(MAKE_SYMBOL_FINISHED, true))) { // if not finished
        for (GoalGroup::const_iterator goal_it = begin(); goal_it != end(); ++goal_it) { // check if all goals are satisfied
            if (!goal_it->is_goal(state)) {
                result = false;
            }
        }
    }
    return result;
}


PredicateGroup GoalGroup::get_predicates() const
{
    PredicateGroup predicates;
    for (GoalGroup::const_iterator goal_it = begin(); goal_it != end(); ++goal_it) {
        predicates.add_predicates(*goal_it);
    }
    return predicates;
}

float GoalGroup::obtain_reward(const PredicateGroup& predicates) const
{
    float obtained_reward = 0;
    for (GoalGroup::const_iterator goal_it = begin(); goal_it != end(); ++goal_it) {
        obtained_reward += goal_it->obtain_reward(predicates);
    }
    return obtained_reward;
}

Goal GoalGroup::adapt_finish_rule_goal_to_simple_predicate_goal()
{
    Goal simple_goal(1.0);

    for (GoalGroup::const_iterator goal_it = begin(); goal_it != end(); ++goal_it) {
        PredicateGroup goal_predicates = *goal_it;
        goal_predicates.remove(Predicate(MAKE_SYMBOL_FINISHED, false));
        if (goal_it->reward < 0.0) {
            goal_predicates = goal_predicates.get_negatet_predicates();
        }
        simple_goal.add_predicates(goal_predicates);
    }

    return simple_goal;
}

void GoalGroup::PPDDL_grounding()
{
    for (GoalGroup::iterator goal_it = begin(); goal_it != end(); ++goal_it) {
        goal_it->PPDDL_grounding();
    }
}



// NON-CLASS FUNCTIONS

PredicateGroup read_predicates_from_file(const std::string file_path)
{
    std::string predicates_string;

    std::ifstream predicates_file(file_path.c_str());
    if (predicates_file.is_open()) {
        if (predicates_file.good()) {
            std::getline(predicates_file, predicates_string);
        }
        predicates_file.close();
    } else LOG(ERROR) << "Unable to open file: " << file_path;

    return PredicateGroup(predicates_string);
}





