/*
 * Class representing a state (as a set of predicates).
 * Copyright (C) 2014  David Martínez (dmartinez@iri.upc.edu)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#ifndef INTELLACT_STATE_H
#define INTELLACT_STATE_H

#include <iterator>
#include "symbolic/predicates.h"
#include "symbolic/rules.h"
#include "symbolic/transitions.h"

/*!
\class State
\brief Class defining the current scenario state.

At this moment it is just a PredicateGroup. In the future it may be enriched with partial observability.
*/
class State : public PredicateGroup
{
public:
    State() {};
    State(const std::string& state_string): PredicateGroup(state_string) {};
    State(const PredicateGroup& pred_group): PredicateGroup(pred_group) {};
    State(const State& state_): PredicateGroup(state_) { };

    void read_from_file(std::string file_path);

    void write_to_file(std::string file_path) const;
    
    void update_predicate(Predicate new_predicate);
    
    /*!
    \brief Removes special predicates such as +-finished() and +-started()
    */
    void remove_special_predicates();

private:

};

typedef boost::shared_ptr<State> StatePtr;
typedef boost::shared_ptr<const State> StateConstPtr;


#endif
