#include <symbolic/symbols.h>


// OPERATORS
bool operator== (const Symbol &s1, const Symbol &s2)
{
    return ((s1.name == s2.name) && utils::compare_ordered_vector_strings(s1.param_names, s2.param_names));
}

bool operator!= (const Symbol &s1, const Symbol &s2)
{
    return !(s1 == s2);
}

bool Symbol::symbolic_compare(const Symbol &s) const
{
    int DEBUG = 0;
    bool equal = (s.name == name);
    if (DEBUG > 2)
        LOG(INFO) << "Symbolic " << *this << ": " << is_symbolic << " - Symbolic " << s << ": " << s.is_symbolic;
    if (is_symbolic || s.is_symbolic) { // if symbolic compare number of params
        equal = (equal && (this->param_names.size() == s.param_names.size()));
    } else { // if grounded compare params content
        equal = (equal && utils::compare_ordered_vector_strings(this->param_names, s.param_names));
    }
    return equal;
}

bool operator<(const Symbol& l, const Symbol& r)
{
    if (l.name < r.name) {
        return true;
    } else if (r.name < l.name) {
        return false;
    } else { //equal
        if (l.param_names.size() < r.param_names.size())
            return true;
        else if (r.param_names.size() < l.param_names.size())
            return false;
        else { // equal number of parameters
            std::vector< std::string >::const_iterator l_it, r_it;
            r_it = r.param_names.begin();
            for (l_it = l.param_names.begin(); l_it != l.param_names.end(); ++l_it) {
                if (*l_it < *r_it) {
                    return true;
                } else if (*r_it < *l_it) {
                    return false;
                }
                ++r_it;
            }
        }
    }
    return false;
}

std::ostream& operator<<(std::ostream &out, const Symbol &s)
{
    out << s.name << "(";
    for (size_t i = 0; i < s.param_names.size(); i++) {
        out << s.param_names[i];
        if ((i + 1) < s.param_names.size())
            out << ",";
    }
    out << ")";
    return out;
}

void Symbol::write_PPDDL(std::ostream &out) const
{
    out << "(" << name;
    for (std::vector<std::string>::const_iterator param_it = param_names.begin(); param_it != param_names.end(); ++param_it) {
        out << " " << *param_it;
    }
    out << ")";
}


// Symbol
Symbol::Symbol(const Symbol &symbol) :
    name(symbol.name),
    param_names(symbol.param_names),
    is_symbolic(symbol.is_symbolic)
{ }

Symbol::Symbol(const std::string &name_, const std::vector<std::string> &param_names_) :
    name(name_),
    param_names(param_names_)
{
    autodetect_symbolic();
}

Symbol::Symbol(const std::string &line):
    name(get_symbol_name_from_string(line)),
    param_names(get_symbol_param_names_from_string(line))
{
    autodetect_symbolic();
}

void Symbol::autodetect_symbolic()
{
    is_symbolic = false;
    for (vector_string::const_iterator param_it = param_names.begin(); param_it != param_names.end(); ++param_it) {
        vector_string::const_iterator find_it = std::find(DEFAULT_PARAM_NAMES_VECTOR.begin(), DEFAULT_PARAM_NAMES_VECTOR.end(), *param_it);
        if (find_it != DEFAULT_PARAM_NAMES_VECTOR.end()) { // found!
            is_symbolic = true;
            return;
        }
    }
}

// bool Symbol::is_empty() const
// {
//     return ((name == "") && (param_names.empty()));
// }

// bool Symbol::has_name(std::string name_str) const
// {
//     if (name_str == name) {
//         //ROS_DEBUG_STREAM("Name comparison is true: " << name );
//         return true;
//     } else
//         return false;
// }

bool Symbol::has_param_with_name(const std::string &param) const
{
    bool result = false;
    for (size_t i = 0; i < param_names.size(); i++) {
        if (param_names[i] == param) {
            result = true;
        }
    }
    return result;
}

int Symbol::get_param_name_idx(const std::string &param) const
{
    int result = -1;
    for (size_t i = 0; i < param_names.size(); i++) {
        if (param_names[i] == param) {
            result = i;
        }
    }
    return result;
}

bool Symbol::has_param_with_all_names(const std::vector<std::string> &params) const
{
    bool result = true;
    for (size_t i = 0; i < param_names.size(); i++) {
        bool present = false;
        for (size_t j = 0; j < params.size(); j++) {
            if (param_names[i] == params[j]) {
                present = true;
            }
        }
        if (!present) {
            result = false;
        }
    }
    return result;
}
/*
bool Symbol::exists() const
{
    return (name.compare("") != 0);
}*/

bool Symbol::is_valid() const
{
    return (name != "");
}

void Symbol::make_symbolic()
{
    for (size_t i = 0; i < param_names.size(); ++i) {
        param_names[i] = DEFAULT_PARAM_NAMES[i];
    }
    is_symbolic = true;
}

bool Symbol::is_special() const
{
    bool result = ((*this == MAKE_SYMBOL_DEAD) || (*this == MAKE_SYMBOL_RETRY) || (*this == MAKE_SYMBOL_HELP) || (*this == MAKE_SYMBOL_UNKNOWN) || (*this == MAKE_SYMBOL_FINISH) || (*this == MAKE_SYMBOL_TEACHER));
    return result;
}

void Symbol::ground(std::vector<std::string> variables)
{
    std::random_shuffle(variables.begin(), variables.end());
    for (size_t i = 0; i < param_names.size(); ++i) {
        if (variables.size() > i) {
            param_names[i] = variables[i];
        } else {
            std::cout << "\033[1;31m" << "Symbols:: ground - Incorrect number of parameters." << "\033[0m" << std::endl;
            std::cout << "\033[1;31m" << "Symbol params are: " << param_names << "\033[0m" << std::endl;
            std::cout << "\033[1;31m" << "Arguments are: " << variables << "\033[0m" << std::endl;
        }
    }
}

void Symbol::PPDDL_grounding()
{
    if (param_names.size() > 0) // first parameter has two _
        name = name + "_";
    for (std::vector<std::string>::iterator param_it = param_names.begin(); param_it != param_names.end(); ++param_it) {
        name = name + "_" + *param_it;
    }
    param_names.clear();
}

/*
Symbol get_symbol_from_string(std::string line)
{
//     line = line.substr(line.find_first_not_of(" \t"));
    line = utils::trim(line);
    Symbol symbol(get_symbol_name_from_string(line), get_symbol_param_names_from_string(line));
    return symbol;
}*/

std::string Symbol::get_symbol_name_from_string(const std::string &line)
{
    std::stringstream ss(line);
    std::string item;

    if (std::getline(ss, item, '(')) {
        //std::cout << "Symbol name: " << item << std::endl;
    }

    return item;
}

std::vector< std::string > Symbol::get_symbol_param_names_from_string(const std::string &line)
{
    std::stringstream ss(line);
    std::string item;
    std::vector< std::string> param_names;

    // get string between ()
    std::getline(ss, item, '(');
    std::getline(ss, item, ')');
    ss.str(item);

    char separator = utils::choose_variable_separator(item);
    while (std::getline(ss, item, separator)) {
        // remove spaces and empty strings
        item.erase(remove_if(item.begin(), item.end(), isspace), item.end());
        if (item.compare("") != 0) {
            param_names.push_back(item);
            //std::cout << "Param names: " << item << std::endl;
        }
    }

    return param_names;
}
